"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformFunctionPrototypeCall = void 0;
const lua = require("../../LuaAST");
const diagnostics_1 = require("../utils/diagnostics");
const function_context_1 = require("../utils/function-context");
const lua_ast_1 = require("../utils/lua-ast");
const lualib_1 = require("../utils/lualib");
const call_1 = require("../visitors/call");
function transformFunctionPrototypeCall(context, node) {
    const expression = node.expression;
    const callerType = context.checker.getTypeAtLocation(expression.expression);
    if (function_context_1.getFunctionContextType(context, callerType) === function_context_1.ContextType.Void) {
        context.diagnostics.push(diagnostics_1.unsupportedSelfFunctionConversion(node));
    }
    const signature = context.checker.getResolvedSignature(node);
    const params = call_1.transformArguments(context, node.arguments, signature);
    const caller = context.transformExpression(expression.expression);
    const expressionName = expression.name.text;
    switch (expressionName) {
        case "apply":
            return lua.createCallExpression(caller, [params[0], lua_ast_1.createUnpackCall(context, params[1], node.arguments[1])], node);
        case "bind":
            return lualib_1.transformLuaLibFunction(context, lualib_1.LuaLibFeature.FunctionBind, node, caller, ...params);
        case "call":
            return lua.createCallExpression(caller, params, node);
        default:
            context.diagnostics.push(diagnostics_1.unsupportedProperty(expression.name, "function", expressionName));
    }
}
exports.transformFunctionPrototypeCall = transformFunctionPrototypeCall;
//# sourceMappingURL=function.js.map